@@ -32,6 +32,7 @@
 #include "ctkconfig.h"
 #include "ctkhelp.h"
 #include "ctkcolorcontrols.h"
+#include "ctkdropdownmenu.h"
 
 /* function prototypes */
 static gboolean build_color_space_table(CtkColorControls *ctk_color_controls,
@@ -43,9 +44,11 @@ static gint map_nvctrl_value_to_table(CtkColorControls *ctk_color_controls,
 static
 gboolean update_color_space_menu_info(CtkColorControls *ctk_color_controls);
 
-static void color_space_menu_changed(GtkOptionMenu *color_space_menu,
+static void setup_reset_button(CtkColorControls *ctk_color_controls);
+
+static void color_space_menu_changed(GtkWidget *widget,
                                      gpointer user_data);
-static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
+static void color_range_menu_changed(GtkWidget *widget,
                                      gpointer user_data);
 
 static void color_control_update_received(GtkObject *object, gpointer arg1,
@@ -90,6 +93,7 @@ GType ctk_color_controls_get_type(void)
             sizeof (CtkColorControls),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_color_controls_type =
@@ -105,24 +109,24 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
                                   CtkConfig *ctk_config,
                                   CtkEvent *ctk_event,
                                   GtkWidget *reset_button,
-                                  unsigned int display_device_mask,
                                   char *name)
 {
     GObject *object;
     CtkColorControls *ctk_color_controls;
     GtkWidget *frame, *hbox, *label;
-    GtkWidget *menu, *table, *menu_item = NULL, *separator;
+    GtkWidget *table, *separator;
+    CtkDropDownMenu *menu;
     ReturnStatus ret1, ret2;
     NVCTRLAttributeValidValuesRec valid1, valid2;
     gint i;
 
     /* check if color configuration is supported */
-    ret1 = NvCtrlGetValidDisplayAttributeValues(handle, display_device_mask,
-                                                NV_CTRL_COLOR_SPACE,
-                                                &valid1);
-    ret2 = NvCtrlGetValidDisplayAttributeValues(handle, display_device_mask,
-                                                NV_CTRL_COLOR_RANGE,
-                                                &valid2);
+    ret1 = NvCtrlGetValidAttributeValues(handle,
+                                         NV_CTRL_COLOR_SPACE,
+                                         &valid1);
+    ret2 = NvCtrlGetValidAttributeValues(handle,
+                                         NV_CTRL_COLOR_RANGE,
+                                         &valid2);
 
     if ((ret1 != NvCtrlSuccess) || (ret2 != NvCtrlSuccess)) {
         return NULL;
@@ -138,7 +142,6 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     ctk_color_controls->handle = handle;
     ctk_color_controls->ctk_config = ctk_config;
     ctk_color_controls->reset_button = reset_button;
-    ctk_color_controls->display_device_mask = display_device_mask;
     ctk_color_controls->name = strdup(name);
 
     /* build a table holding available color space */
@@ -150,7 +153,7 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, FRAME_PADDING);
-    ctk_color_controls->color_controls_main = hbox;
+    ctk_color_controls->color_controls_box = hbox;
 
     frame = gtk_frame_new("Color Controls");
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
@@ -162,29 +165,24 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 
     /* dropdown list for color space */
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
 
     for (i = 0; i < ctk_color_controls->color_space_table_size; i++) {
         switch (ctk_color_controls->color_space_table[i]) {
         case NV_CTRL_COLOR_SPACE_YCbCr422:
-            menu_item = gtk_menu_item_new_with_label("YCbCr422");
+            ctk_drop_down_menu_append_item(menu, "YCbCr422", i);
             break;
         case NV_CTRL_COLOR_SPACE_YCbCr444:
-            menu_item = gtk_menu_item_new_with_label("YCbCr444");
+            ctk_drop_down_menu_append_item(menu, "YCbCr444", i);
             break;
         default:
         case NV_CTRL_COLOR_SPACE_RGB:
-            menu_item = gtk_menu_item_new_with_label("RGB");
+            ctk_drop_down_menu_append_item(menu, "RGB", i);
             break;
         }
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
     }
-
-    ctk_color_controls->color_space_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         menu);
+    ctk_color_controls->color_space_menu = GTK_WIDGET(menu);
     ctk_config_set_tooltip(ctk_config, 
                            ctk_color_controls->color_space_menu,
                            __color_space_help);
@@ -210,7 +208,6 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
     hbox = gtk_hbox_new(FALSE, 0);
-    ctk_color_controls->color_space_box = hbox;
     gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_box_pack_start(GTK_BOX(hbox),
@@ -226,8 +223,10 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 
     /* Build color widgets & pack them in table */
     /* dropdown list for color range */
-
-    ctk_color_controls->color_range_menu = gtk_option_menu_new();
+    
+    ctk_color_controls->color_range_menu =
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+    
     ctk_config_set_tooltip(ctk_config, 
                            ctk_color_controls->color_range_menu,
                            __color_range_help);
@@ -270,6 +269,53 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 } /* ctk_color_controls_new() */
 
 
+
+/*
+ * setup_reset_button() - enables the reset button if any of the current
+ * settings are not the default.
+ */
+static void setup_reset_button(CtkColorControls *ctk_color_controls)
+{
+    gint history;
+    gint val;
+    CtkDropDownMenu *color_space_menu, *color_range_menu;
+
+    if (!GTK_WIDGET_SENSITIVE(ctk_color_controls->color_controls_box)) {
+        /* Nothing is available, don't bother enabling the reset button yet. */
+        return;
+    }
+
+    /* The color space menu is always available */
+    color_space_menu = CTK_DROP_DOWN_MENU(ctk_color_controls->color_space_menu);
+    history = color_space_menu->current_selected_item;
+
+    val = ctk_color_controls->color_space_table[history];
+    if (val != NV_CTRL_COLOR_SPACE_RGB) {
+        goto enable;
+    }
+
+    /* Color range is dependent on the color space */
+    if (GTK_WIDGET_SENSITIVE(ctk_color_controls->color_range_menu)) {
+        color_range_menu = 
+            CTK_DROP_DOWN_MENU(ctk_color_controls->color_range_menu);
+        history = color_range_menu->current_selected_item; 
+        val = ctk_color_controls->color_range_table[history];
+        if (val != NV_CTRL_COLOR_RANGE_FULL) {
+            goto enable;
+        }
+    }
+
+    /* Don't disable reset button here, since other settings that are not
+     * managed by the ctk_image_slider here may need it enabled
+     */
+    return;
+
+ enable:
+    gtk_widget_set_sensitive(ctk_color_controls->reset_button, TRUE);
+}
+
+
+
 /*
  * ctk_color_controls_setup() - Setup routine for color attributes. Used
  * in DFP setup stage as well as for updating the GUI when there is change in
@@ -283,10 +329,12 @@ void ctk_color_controls_setup(CtkColorControls *ctk_color_controls)
 
     /* color space */
     if (!update_color_space_menu_info(ctk_color_controls)) {
-        gtk_widget_set_sensitive(ctk_color_controls->color_controls_main,
-                                 FALSE);
-        gtk_widget_hide_all(ctk_color_controls->color_controls_main);
+        gtk_widget_set_sensitive(ctk_color_controls->color_controls_box, FALSE);
+        gtk_widget_hide_all(ctk_color_controls->color_controls_box);
     }
+
+    setup_reset_button(ctk_color_controls);
+
 } /* ctk_color_controls_setup() */
 
 
@@ -296,11 +344,9 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
 
     /* color space */
     if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_color_controls->handle,
-                                  ctk_color_controls->display_device_mask,
-                                  NV_CTRL_COLOR_SPACE,
-                                  &color_space)) {
-        free(ctk_color_controls->color_space_table);
+        NvCtrlGetAttribute(ctk_color_controls->handle,
+                           NV_CTRL_COLOR_SPACE,
+                           &color_space)) {
         return FALSE;
     }
 
@@ -312,9 +358,8 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
          G_CALLBACK(color_space_menu_changed),
          (gpointer) ctk_color_controls);
 
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         color_space);
+    ctk_drop_down_menu_set_current_value
+        (CTK_DROP_DOWN_MENU(ctk_color_controls->color_space_menu), color_space);
 
     g_signal_handlers_unblock_by_func
         (G_OBJECT(ctk_color_controls->color_space_menu),
@@ -322,9 +367,14 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
          (gpointer) ctk_color_controls);
 
     /* dynamically regenerate color range dropdown */
-    setup_color_range_dropdown(ctk_color_controls);
+    if (!setup_color_range_dropdown(ctk_color_controls)) {
+        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, FALSE);
+    } else {
+        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, TRUE);
+    }
+
+    return TRUE;
 
-    return True;
 } /* update_color_space_menu_info() */
 
 static
@@ -360,34 +410,20 @@ void post_color_space_update(CtkColorControls *ctk_color_controls,
                                  ctk_color_controls->name);
 }
 
-static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
+static void color_range_menu_changed(GtkWidget *widget,
                                      gpointer user_data)
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_range = NV_CTRL_COLOR_RANGE_FULL;
 
-    history = gtk_option_menu_get_history(color_range_menu);
+    history = ctk_drop_down_menu_get_current_value(menu);
     color_range = ctk_color_controls->color_range_table[history];
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_RANGE,
-                              color_range);
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         color_range);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_RANGE,
+                       color_range);
 
     /* reflecting the change to statusbar message and the reset button */
     post_color_range_update(ctk_color_controls, color_range);
@@ -395,37 +431,24 @@ static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
 } /* color_range_menu_changed() */
 
 
-static void color_space_menu_changed(GtkOptionMenu *color_space_menu,
+static void color_space_menu_changed(GtkWidget *widget,
                                      gpointer user_data)
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_space = NV_CTRL_COLOR_SPACE_RGB;
-
-    history = gtk_option_menu_get_history(color_space_menu);
+    
+    history = ctk_drop_down_menu_get_current_value(menu);
 
     color_space = ctk_color_controls->color_space_table[history];
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_SPACE,
-                              color_space);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_SPACE,
+                       color_space);
 
     color_space = map_nvctrl_value_to_table(ctk_color_controls,
                                             color_space);
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_space_menu),
-         G_CALLBACK(color_space_menu_changed),
-         (gpointer) ctk_color_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         color_space);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_color_controls->color_space_menu),
-         G_CALLBACK(color_space_menu_changed),
-         (gpointer) ctk_color_controls);
 
     /* reflecting the change in color space to other widgets & reset button */
     ctk_color_controls_setup(ctk_color_controls);
@@ -444,15 +467,13 @@ void ctk_color_controls_reset(CtkColorControls *ctk_color_controls)
         return;
     }
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_SPACE,
-                              NV_CTRL_COLOR_SPACE_RGB);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_SPACE,
+                       NV_CTRL_COLOR_SPACE_RGB);
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_RANGE,
-                              NV_CTRL_COLOR_RANGE_FULL);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_RANGE,
+                       NV_CTRL_COLOR_RANGE_FULL);
 
     ctk_color_controls_setup(ctk_color_controls);
 } /* ctk_color_controls_reset() */
@@ -465,6 +486,10 @@ void add_color_controls_help(CtkColorControls *ctk_color_controls,
                              GtkTextBuffer *b,
                              GtkTextIter *i)
 {
+    if (!ctk_color_controls) {
+        return;
+    }
+
     ctk_help_heading(b, i, "Color Controls");
     ctk_help_para(b, i, __color_controls_help);
 
@@ -487,14 +512,8 @@ static void color_control_update_received(GtkObject *object, gpointer arg1,
     CtkColorControls *ctk_object = CTK_COLOR_CONTROLS(user_data);
     CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
 
-    /* if the event is not for this display device, return */
-
-    if (!(event_struct->display_mask & ctk_object->display_device_mask)) {
-        return;
-    }
-
     ctk_color_controls_setup(ctk_object);
-    
+
     /* update status bar message */
     switch (event_struct->attribute) {
     case NV_CTRL_COLOR_RANGE:
@@ -554,13 +573,15 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
     gint i, n = 0, color_range_count = 0;
     gint mask, val;
     ReturnStatus ret;
-    GtkWidget *menu, *menu_item;
     NVCTRLAttributeValidValuesRec valid;
+    CtkDropDownMenu *d;
 
-    ret = NvCtrlGetValidDisplayAttributeValues(ctk_color_controls->handle,
-                                               ctk_color_controls->display_device_mask,
-                                               NV_CTRL_COLOR_RANGE,
-                                               &valid);
+    ret = NvCtrlGetValidAttributeValues(ctk_color_controls->handle,
+                                        NV_CTRL_COLOR_RANGE,
+                                        &valid);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
 
     if (valid.type != ATTRIBUTE_TYPE_INT_BITS) {
         return FALSE;
@@ -591,55 +612,47 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
     }
     
     /* dropdown list for color range */
-    menu = gtk_menu_new();
+    d = (CtkDropDownMenu *) ctk_color_controls->color_range_menu;
+
+    g_signal_handlers_block_by_func
+         (G_OBJECT(ctk_color_controls->color_range_menu),
+         G_CALLBACK(color_range_menu_changed),
+         (gpointer) ctk_color_controls);
+    
+    ctk_drop_down_menu_reset(d);
 
     for (i = 0; i < ctk_color_controls->color_range_table_size; i++) {
         switch (ctk_color_controls->color_range_table[i]) {
         case NV_CTRL_COLOR_RANGE_FULL:
-            menu_item = gtk_menu_item_new_with_label("Full");
+            ctk_drop_down_menu_append_item(d, "Full", i);
             break;
         default:
         case NV_CTRL_COLOR_RANGE_LIMITED:
-            menu_item = gtk_menu_item_new_with_label("Limited");
+            ctk_drop_down_menu_append_item(d, "Limited", i);
             break;
         }
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
     }
 
     /* color range */
     if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_color_controls->handle,
-                                  ctk_color_controls->display_device_mask,
-                                  NV_CTRL_COLOR_RANGE, &val)) {
+        NvCtrlGetAttribute(ctk_color_controls->handle,
+                           NV_CTRL_COLOR_RANGE,
+                           &val)) {
         val = NV_CTRL_COLOR_RANGE_FULL;
     }
 
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
+    ctk_drop_down_menu_set_current_value(d, val);
 
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         menu);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         val);
-
-    /* If dropdown only has one item, disable it */
-    if (ctk_color_controls->color_range_table_size > 1) {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, True);
-    } else {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, False);
-    }
-    
     g_signal_handlers_unblock_by_func
         (G_OBJECT(ctk_color_controls->color_range_menu),
          G_CALLBACK(color_range_menu_changed),
          (gpointer) ctk_color_controls);
 
+    /* If dropdown only has one item, disable it */
+    if (ctk_color_controls->color_range_table_size <= 1) {
+        return FALSE;
+    }
+
     return TRUE;
 
 } /* setup_color_range_dropdown() */
